@@ -80,48 +80,6 @@ us <- function(x, split="", ...) { unlist( strsplit( x, split=split, ...) ) }
   return( !(x %in% y) )
 }
 
-#' Test Objects for Exact Equality
-#' 
-#' The safe and reliable way to test two objects for being exactly equal.
-#' Wrapper to \code{\link{identical}}.
-#' 
-#' @param x an \R object.
-#' @param y an \R object.
-#' @param ... additional arguments passed to \code{\link{identical}}.
-"%identical%" <- function(x, y, ...) {
-  return( identical(x, y, ...) )
-}
-
-#' Inverse grep
-#' 
-#' From a vector \code{x}, return the indices of elements that do not
-#' match \code{pattern}.
-#' 
-#' Use \code{value=TRUE} to return the actual non-matched elements.
-#' 
-#' @param pattern regular expression pattern
-#' @param x vector of items to match against
-#' @param perl boolean. use perl-compatible regular expressions?
-#' @param value logical. if \code{TRUE}, we return the actual values matched;
-#' otherwise we return the indices of the matches.
-#' @param ... passed to grep
-#' @return the elements in \code{x} that did not match the pattern supplied
-#' @seealso \code{\link{grep}} \code{\link{regex}}
-#' @export
-#' @examples
-#' some_files <- c("output_file.tar.gz", "old_log1.txt", "old_log2.txt")
-#' ## get the non-log files
-#' files_I_want <- ngrep( "_log[0-9]+\\.txt$", some_files )
-ngrep <- function(pattern, x, perl=TRUE, value=FALSE, ...) {
-  if( isTRUE(value) ) {
-    x[ x %nin% grep(pattern, x, value=TRUE, perl=perl, ...) ]
-  } else {
-    tmp <- 1:length(x)
-    return( tmp[ tmp %nin% grep(pattern, x, value=FALSE, perl=perl, ...)])
-  }
-}
-  
-
 #' Extract Elements from a Named Object
 #' 
 #' Extracts elements from an \R object
@@ -167,16 +125,11 @@ extract <- function( x, ... ) {
           )
       })
       return(tmp)
-      
     }
-    
     return(xx)
-    
   })
-  
   x <- x[ names(x) %in% unlist(args) ]
   return( x )
-  
 }
 
 #' Extract Elements from a Named Object with Regular Expressions
@@ -403,7 +356,8 @@ write.cb <- function( dat,
 #' This function writes data to the clipboard, using \code{\link{cat}}.
 #' @param dat data to be written to the clipboard.
 #' @param ... optional arguments passed to \code{cat}.
-#' @seealso \code{\link{write.cb}}
+#' @seealso \code{\link{write.cb}}, \code{\link{cat}}
+#' @export
 cat.cb <- function( dat, ... ) {
   if( Sys.info()["sysname"] == "Darwin" ) {
     cat( dat, file=pipe("pbcopy"), ... )
@@ -420,21 +374,31 @@ cat.cb <- function( dat, ... ) {
 #' - the primary bonus is the automatic coersion to a \code{data.frame}.
 #' @param x a vector of strings.
 #' @param sep the delimiter / \code{\link{regex}} you wish to split your strings on.
+#' @param fixed logical. If \code{TRUE}, we match \code{sep} exactly; 
+#' otherwise, we use regular expressions. Has priority over \code{perl}.
+#' @param perl logical. Should perl-compatible regexps be used?
+#' @param useBytes logical. If \code{TRUE}, matching is done byte-by-byte rather than
+#' character-by-character.
 #' @param names optional: a vector of names to pass to the returned \code{data.frame}.
-#' @param ... optional arguments passed to \code{strsplit}.
 #' @seealso \code{\link{strsplit}}
 #' @export
 #' @examples
-#' dat <- str_split( 
+#' str_split( 
 #'   c("regular_structure", "in_my", "data_here"), 
 #'   sep="_", 
 #'   names=c("apple", "banana") 
 #' )
 #' x <- c("somewhat_different.structure", "in_this.guy")
-#' str_split( x, "[_\\.]" )
-str_split <- function(x, sep, names=NULL, ...) {
+#' str_split( x, "[_\\.]", names=c("first", "second", "third") )
+str_split <- function(x, sep, fixed=FALSE, perl=TRUE, useBytes=FALSE, names=NULL) {
+  
   x <- as.character(x)
-  tmp <- unlist( strsplit( x, sep, ... ) )
+  
+  tmp <- strsplit( x, sep, fixed=fixed, perl=perl, useBytes=useBytes )
+  if( length( unique( unlist( lapply( tmp, length ) ) ) ) > 1 ) {
+    stop("non-equal lengths for each entry of x post-splitting")
+  }
+  tmp <- unlist( tmp )
   tmp <- as.data.frame( 
     matrix( tmp, ncol = (length(tmp) / length(x)), byrow=T ),
     stringsAsFactors=FALSE, optional=TRUE 
@@ -467,7 +431,8 @@ str_split <- function(x, sep, names=NULL, ...) {
 #' swap( x, from, to )
 #' 
 #' ## alternatively, we can submit a named character vector
-#' ## we translate from value to name
+#' ## we translate from value to name. note that this forces
+#' ## a conversion to character
 #' names(from) <- to
 #' swap( x, from )
 #' 
@@ -476,80 +441,49 @@ str_split <- function(x, sep, names=NULL, ...) {
 #' swap(x, c(1, 2), c("a", "b") )
 #' 
 swap <- function( vec, from, to=names(from), ... ) {
-  if( all( as.character(to) == names(from) ) && 
-        is.numeric(vec) && 
-        all( names(from) == as.numeric( names(from) ) ) ) {
-    tmp <- as.numeric( to[ match( vec, from, ... ) ] )
-  } else {
-    tmp <- to[ match( vec, from, ... ) ]
-  }
-  
+  tmp <- to[ match( vec, from, ... ) ]
   tmp[ is.na(tmp) ] <- vec[ is.na(tmp) ]
-  
   return( tmp )
 }
 
-#' Stack a List of DataFrame-like Objects
+#' Converts Factors to Characters in an Object
 #' 
-#' Function for stacking a list, where each component of the list
-#' is a \code{data.frame} or a \code{matrix} containing potentially differing 
-#' number of rows, but the same columns. The main 'extra' is handling of 
-#' row names, which are passed on into the stacked data frame. 
-#' These are passed into a column called \code{which} to protect from 
-#' problems with non-unique row names, and also to avoid appending 
-#' numbers onto these row names as well.
-#' @param list a list of data frames.
-#' @param which boolean. add a column \code{which} built from row names?
+#' Converts factors to characters in an object. Leaves non-character
+#' elements untouched.
+#' 
+#' We iterate through all elements in the object (e.g. if
+#' it is a list) and convert anything that is a factor into a character.
+#' @param X an object.
 #' @export
-#' @examples
-#' x <- data.frame( x=c(1, 2, 3) )
-#' rownames(x) <- c("apple", "banana", "cherry")
-#' y <- data.frame( x=c('a', 'b', 'c') )
-#' rownames(y) <- c("date", "eggplant", "fig")
-#' stack_list( list(x, y) )
-stack_list <- function( list, which=TRUE ) {
-  stopifnot( all( sapply( list, class ) %in% c("data.frame", "matrix") ) )
-  tmp <- do.call( rbind, list )
-  if( "which" %in% colnames(tmp) )
-    stop("ERROR: 'which' is already a column name in your data!")
-  if( isTRUE(which) ) { 
-    tmp$which <- gsub( "\\.[0-9]+$", "", rownames( tmp ))
-  }
-  rownames(tmp) <- NULL
-  return(tmp)  
+factor_to_char <- function( X ) {
+  return( .Call("factor_to_char", X, PACKAGE="Kmisc") )
 }
 
 #' Converts Characters to Factors in an Object
 #' 
-#' Converts characters to factors in an object. Leaves non-character
-#' elements untouched. Objects that can be subscripted with \code{[[}
-#' are supported.
-#' @param x an object.
-#' @param ... optional arguments passed to \code{\link{factor}}.
+#' Converts characters to factors in an object. Leaves non-factor elements
+#' untouched.
+#' @param X an object.
+#' @param ... optional arguments passed to \code{factor}.
 #' @export
-char_to_factor <- function( x, ... ) {
-  for( i in 1:length(x) ) {
-    if( is.character( x[[i]] ) ) {
-      x[[i]] <- factor( x[[i]], ... )
-    } 
-  }
-  return(x)
-}
+char_to_factor <- function(X, ...) {
   
-#' Converts Factors to Characters in an Object
-#' 
-#' Converts factors to characters in an object. Leaves non-factor elements
-#' untouched. Objects that can be subscripted with \code{[[}
-#' are supported.
-#' @param x an object.
-#' @export
-factor_to_char <- function(x) {
-  for( i in 1:length(x) ) {
-    if( is.factor( x[[i]] ) ) {
-      x[[i]] <- as.character( x[[i]] )
-    } 
+  if( is.list(X) ) {
+    return( rapply( X, how="replace", function(x) {
+      if( is.character(x) ) {
+        return( factor(x, ...) )
+      } else {
+        return( x )
+      }
+    }))
+  } else {
+    if( is.character(X) ) {
+      return( factor(X, ...) )
+    } else {
+      return( X )
+    }
   }
-  return(x)
+  
 }
 
 #' Make Dummy Variables from a Factor
@@ -644,8 +578,8 @@ strip_extension <- function(x, lvl=1) {
 #' saving that object. Often, we would prefer to assign the \code{load}ed 
 #' object to a new variable name. Hence, this function.
 #' 
-#' If multiple arguments are supplied, they will be \code{paste}d together,
-#' with \code{collapse=""}.
+#' If multiple arguments are supplied, they will be concatenated through
+#' \code{file.path}.
 #' @param ... args to pass to \code{load}
 #' @seealso \code{\link{load}}
 #' @export
@@ -658,9 +592,13 @@ strip_extension <- function(x, lvl=1) {
 #' ## we protect ourselves from 'forgetting' the name of the
 #' ## object we saved
 kLoad <- function( ... ) {
-  get( load( paste(..., sep="", collapse="") ) )
+  get( load( file.path( ... ) ) )
 }
 
+#' @rdname kLoad
+#' @export
+getload <- kLoad
+
 #' Write out and Save a Tabular File
 #' 
 #' A function that both writes a file to table with \code{write.table},
@@ -691,7 +629,7 @@ kSave <- function( x, file, lvl=1, Rext=".rda", ... ) {
   
 }
 
-#' Merge (Left Join) with Index Retainment
+#' Merge (Left Join) with Order Retainment
 #' 
 #' \code{merge} will mangle the order of the data frames it is merging. This is
 #' a simple modification to ensure that the order in data frame \code{x} is preserved
@@ -700,10 +638,7 @@ kSave <- function( x, file, lvl=1, Rext=".rda", ... ) {
 #' data frame \code{y}, we can merge in the parts of \code{y} whose index matches
 #' with that of \code{x}, while preserving the ordering of \code{x}.
 #' 
-#' The function requires you to specify the \code{by} argument; ie, you must have
-#' a shared column in your data frames \code{x} and \code{y}.
-#' 
-#' @param x the \code{data.frame} you wish to merge y into.
+#' @param x the \code{data.frame} you wish to merge \code{y} into.
 #' @param y the \code{data.frame} to be merged.
 #' @param by specifications of the columns used for merging. See 'Details' of \code{\link{merge}}.
 #' @param by.x specifications of the columns used for merging. See 'Details' of \code{\link{merge}}.
@@ -724,12 +659,14 @@ kSave <- function( x, file, lvl=1, Rext=".rda", ... ) {
 #' kMerge(x, y, by="id")
 kMerge <- function( x, y, by=intersect( names(x), names(y) ), by.x=by, by.y=by, ... ) {
   
-  kName <- paste(by, "_orig", sep="")
-  x[,kName] <- 1:nrow(x)
-  tmp <- merge( x=x, y=y, by=by, all.x = TRUE, ... )
-  tmp <- tmp[order(tmp[,kName]),]
-  tmp <- tmp[,!(names(tmp) %in% kName)]
-  tmp
+  x[["__KMERGE_TMP__"]] <- 1:nrow(x)
+  tmp <- merge( x=x, y=y, by=by, by.x=by.x, by.y=by.y, all.x = TRUE, ... )
+  tmp <- tmp[order(tmp[["__KMERGE_TMP__"]]),]
+  tmp <- tmp[,!(names(tmp) %in% "__KMERGE_TMP__")]
+  if( !is.null( attr( x, "row.names" ) ) ) {
+    attr( tmp, "row.names" ) <- attr( x, "row.names" )
+  }
+  return( tmp )
   
 }
 
@@ -1113,14 +1050,14 @@ kCoef <- function( fit,
 #' Returns a nicely formatted ANOVA table. 
 #' See \code{\link{kCoef}} for other details.
 #' @param fit the model fit to generate an ANOVA table for.
-#' @param test the type of test to perform. 
-#' default is likelihood-ratio test (LRT).
+#' @param test the type of test to perform. default is likelihood-ratio test (LRT).
 #' @param swap.periods swap periods with spaces?
 #' @export
 #' @examples
 #' x <- rnorm(100)
 #' y <- ifelse( x + runif(100) > 1, 1, 0 )
 #' myFit <- glm( y ~ x, family="binomial" )
+#' kAnova( myFit )
 kAnova <- function( fit, test="LRT", swap.periods=TRUE ) {
   
   f <- function(x) {
